// Spart License (zlib/png)
// 
// 
// Copyright (c) 2003 Jonathan de Halleux
// 
// This software is provided 'as-is', without any express or implied warranty. 
// In no event will the authors be held liable for any damages arising from 
// the use of this software.
// 
// Permission is granted to anyone to use this software for any purpose, 
// including commercial applications, and to alter it and redistribute it 
// freely, subject to the following restrictions:
// 
// 1. The origin of this software must not be misrepresented; you must not 
// claim that you wrote the original software. If you use this software in a 
// product, an acknowledgment in the product documentation would be 
// appreciated but is not required.
// 
// 2. Altered source versions must be plainly marked as such, and must not be 
// misrepresented as being the original software.
// 
// 3. This notice may not be removed or altered from any source distribution.
// 
// Author: Jonathan de Halleuxusing System;

namespace Spart.Parsers
{
	/// <summary>
	/// A parser that can be negated (will match anything that it would not have matched and won't match anything it would have matched)
	/// </summary>
    abstract class NegatableParser : TerminalParser
	{
		private bool m_Negate=false;

        /// <summary>
        /// Should this parser be negated
        /// </summary>
		public bool Negate 
		{
			get
			{
				return m_Negate;
			}
			set
			{
				if( m_Negate != value )
					m_Negate = value;
			}
		}

		/// <summary>
		/// Negate this parser
		/// </summary>
		/// <param name="p">the parser to negate</param>
		/// <returns></returns>
        public static NegatableParser operator ~ (NegatableParser p)
		{
			p.Negate=!p.Negate;
			return p;
		}
	}
}
